using System;
using System.Diagnostics;
using System.ServiceProcess;
using Microsoft.Win32;
using System.IO;
using System.Security.Principal;
using System.DirectoryServices;
using System.Collections;
using System.Threading;

namespace WindowsService
{
    class WindowsService : ServiceBase
    {
        private RegistryMonitor[] m_RegistryMonitorArr = null;
        private bool              m_IsEndMyUpdate      = false;
        private ArrayList         m_UsersKeys          = new ArrayList();

        /// <summary>
        /// Public Constructor for WindowsService.
        /// - Put all of your Initialization code here.
        /// </summary>
        public WindowsService()
        {
            this.ServiceName = "Distartup";
            this.EventLog.Source = "Distartup";
            this.EventLog.Log = "Application";
            
            // These Flags set whether or not to handle that specific
            //  type of event. Set to true if you need it, false otherwise.
            this.CanHandlePowerEvent = true;
            this.CanHandleSessionChangeEvent = true;
            this.CanPauseAndContinue = true;
            this.CanShutdown = true;
            this.CanStop = true;

            if (!EventLog.SourceExists("Distartup"))
            {
                EventLog.CreateEventSource("Distartup", "Application");
            }
        }

        /// <summary>
        /// The Main Thread: This is where your Service is Run.
        /// </summary>
        static void Main()
        {
            ServiceBase.Run(new WindowsService());
        }

        /// <summary>
        /// Dispose of objects that need it here.
        /// </summary>
        /// <param name="disposing">Whether or not disposing is going on.</param>
        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }

        /// <summary>
        /// OnStart: Put startup code here
        ///  - Start threads, get inital data, etc.
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            base.OnStart(args);
            DeleteFromRegistry();
            RegistryMonitor();
        }

        /*
        private void CheckWinlogon()
        {
            while (true)
            {
                Process[] list = Process.GetProcessesByName("explorer");

                if (list != null && list.Length > 0)
                {
                    break;
                }

                Thread.Sleep(1000);
            }

            RunPrograms();
        }
        
        private void RunPrograms()
        {
            string drive = Environment.GetEnvironmentVariable("systemdrive");
            string fileName = drive + "\\startup.programs.txt";

            try
            {
                if (File.Exists(fileName))
                {
                    using (StreamReader sr = new StreamReader(fileName))
                    {
                        while (sr.Peek() >= 0)
                        {
                            sr.ReadLine();
                        }
                    }
                }
            }
            catch (Exception e)
            {
                EventLog.WriteEntry("Exception", e.StackTrace, EventLogEntryType.Error);
            }
        }
        */
        
        /// <summary>
        /// Delete sub tree of registry entry
        /// </summary>
        /// <param name="key"></param>
        protected void DeleteTree(RegistryKey key)
        {
            string[] tree = key.GetSubKeyNames();
            string[] values = key.GetValueNames();

            foreach (string temp in tree)
            {
                key.DeleteSubKeyTree(temp);
                EventLog.WriteEntry("Distartup", "Delete " + temp, EventLogEntryType.Information);
            }

            foreach (string temp in values)
            {
                key.DeleteValue(temp);
                EventLog.WriteEntry("Distartup", "Delete " + temp, EventLogEntryType.Information);
            }
        }

        /// <summary>
        /// Delete from HKEY_LOCAL_MACHINE the startup entries
        /// </summary>
        private void DeleteFromRegistry()
        {
              try
              {
                  /********************************************************************************************/
                  RegistryKey key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Run",true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }

                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }

                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\RunOnceEx", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }
                  /********************************************************************************************/


                  /********************************************************************************************/
                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows NT\\CurrentVersion\\Run", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }

                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows NT\\CurrentVersion\\RunOnce", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }

                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Windows NT\\CurrentVersion\\RunOnceEx", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }
                  /********************************************************************************************/
                  
                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Shared Tools\\MSConfig\\startupreg", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }
                  /********************************************************************************************/
                  key = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\Shared Tools\\MSConfig\\startupfolder", true);
                  if (key != null)
                  {
                      DeleteTree(key);
                  }
                  /********************************************************************************************/

                  DeleteUsersKeys();

                  m_IsEndMyUpdate = true;
                  /********************************************************************************************/


                  /********************************************************************************************/
                  string allusersprofile = Environment.GetEnvironmentVariable("allusersprofile");

                  if (allusersprofile != null)
                  {
                      DeleteFiles(allusersprofile + "\\Start Menu\\Programs\\Startup\\");
                  }
                  /********************************************************************************************/


                  /********************************************************************************************/
                  DeleteFromUsers();
                  /********************************************************************************************/
                        
              }
              catch (Exception ex)
              {
                  EventLog.WriteEntry("Exception", ex.StackTrace, EventLogEntryType.Error);
              }
        }

        /// <summary>
        /// Delete from users reg the startup programs
        /// </summary>
        private void DeleteUsersKeys()
        {
            string[] subkeys = Registry.Users.GetSubKeyNames();

            if (subkeys != null)
            {
                foreach (string temp in subkeys)
                {
                    RegistryKey userskeyOne = Registry.Users.OpenSubKey(temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                    RegistryKey userskeyTwo = Registry.Users.OpenSubKey(temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce", true);

                    if (userskeyOne != null)
                    {
                        DeleteTree(userskeyOne);
                    }
                    else 
                    {
                        //EventLog.WriteEntry("Distartup", temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\Run is null", EventLogEntryType.Warning);
                    }

                    if (userskeyTwo != null)
                    {
                        DeleteTree(userskeyTwo);
                    }
                    else
                    {
                        //EventLog.WriteEntry("Distartup", temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce is null", EventLogEntryType.Warning);
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private void RegistryMonitor()
        {
            m_UsersKeys.Add("HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Run");
            m_UsersKeys.Add("HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce");
            m_UsersKeys.Add("HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\RunOnceEx");
            
            string[] subkeys = Registry.Users.GetSubKeyNames();

            if (subkeys != null)
            {
                foreach (string temp in subkeys)
                {
                    RegistryKey userskeyOne = Registry.Users.OpenSubKey(temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                    RegistryKey userskeyTwo = Registry.Users.OpenSubKey(temp + "\\Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce", true);

                    if (userskeyOne != null)
                    {
                        m_UsersKeys.Add(userskeyOne.ToString());
                    }

                    if (userskeyTwo != null)
                    {
                        m_UsersKeys.Add(userskeyTwo.ToString());
                    }
                }
            }

            m_RegistryMonitorArr = new RegistryMonitor[m_UsersKeys.Count];
            int i = 0;

            foreach (string key in m_UsersKeys)
            {
                m_RegistryMonitorArr[i] = new RegistryMonitor(key);
                m_RegistryMonitorArr[i].RegChanged += new EventHandler(OnRegChanged);
                m_RegistryMonitorArr[i].Error += new System.IO.ErrorEventHandler(OnError);
                m_RegistryMonitorArr[i].Start();
                ++i;
            }
        }

        private void OnRegChanged(object sender, EventArgs e)
        {
            if (m_IsEndMyUpdate)
            {
                m_IsEndMyUpdate = false;
                EventLog.WriteEntry("Distartup", "Registry changed", EventLogEntryType.Information);
                DeleteFromRegistry();
            }
        }

        private void OnError(object sender, ErrorEventArgs e)
        {
            EventLog.WriteEntry("Distartup", e.GetException().Message, EventLogEntryType.Error);
        }

        private void StopRegistryMonitor()
        {
            EventLog.WriteEntry("Distartup", "Service stopped", EventLogEntryType.Warning);

            if (m_RegistryMonitorArr != null)
            {
                for (int i = 0; i < m_RegistryMonitorArr.Length; ++i)
                {
                    m_RegistryMonitorArr[i].Stop();
                    m_RegistryMonitorArr[i].RegChanged -= new EventHandler(OnRegChanged);
                    m_RegistryMonitorArr[i].Error -= new System.IO.ErrorEventHandler(OnError);
                    m_RegistryMonitorArr[i] = null;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        protected void DeleteFromUsers()
        {
            string drive = Environment.GetEnvironmentVariable("systemdrive");
            string start = "\\Documents and Settings\\";
            string end = "\\Start Menu\\Programs\\Startup\\";
            
            if (drive != null)
            {
                DirectoryEntry localMachine = new DirectoryEntry("WinNT://" + Environment.MachineName);
                DirectoryEntry admGroup = localMachine.Children.Find("administrators", "group");
                object members = admGroup.Invoke("members", null);
                foreach (object groupMember in (IEnumerable)members)
                {
                    DirectoryEntry member = new DirectoryEntry(groupMember);

                    if (!member.Name.Equals("Administrator"))
                    {
                        DeleteFiles(drive + start + member.Name + end);
                    }
                }
            }
        }

        protected void DeleteFiles(string path)
        {
            EventLog.WriteEntry("Distartup", path, EventLogEntryType.Information);
            string[] files = Directory.GetFiles(path);
            foreach (string file in files)
            {
                File.Delete(file);
            }
        }

        /// <summary>
        /// OnStop: Put your stop code here
        /// - Stop threads, set final data, etc.
        /// </summary>
        protected override void OnStop()
        {
            base.OnStop();
            //StopRegistryMonitor();
        }

        /// <summary>
        /// OnPause: Put your pause code here
        /// - Pause working threads, etc.
        /// </summary>
        protected override void OnPause()
        {
            base.OnPause();
        }

        /// <summary>
        /// OnContinue: Put your continue code here
        /// - Un-pause working threads, etc.
        /// </summary>
        protected override void OnContinue()
        {
            base.OnContinue();
        }

        /// <summary>
        /// OnShutdown(): Called when the System is shutting down
        /// - Put code here when you need special handling
        ///   of code that deals with a system shutdown, such
        ///   as saving special data before shutdown.
        /// </summary>
        protected override void OnShutdown()
        {
            base.OnShutdown();
        }

        /// <summary>
        /// OnCustomCommand(): If you need to send a command to your
        ///   service without the need for Remoting or Sockets, use
        ///   this method to do custom methods.
        /// </summary>
        /// <param name="command">Arbitrary Integer between 128 & 256</param>
        protected override void OnCustomCommand(int command)
        {
            //  A custom command can be sent to a service by using this method:
            //#  int command = 128; //Some Arbitrary number between 128 & 256
            //#  ServiceController sc = new ServiceController("NameOfService");
            //#  sc.ExecuteCommand(command);

            base.OnCustomCommand(command);
        }

        /// <summary>
        /// OnPowerEvent(): Useful for detecting power status changes,
        ///   such as going into Suspend mode or Low Battery for laptops.
        /// </summary>
        /// <param name="powerStatus">The Power Broadcase Status (BatteryLow, Suspend, etc.)</param>
        protected override bool OnPowerEvent(PowerBroadcastStatus powerStatus)
        {
            return base.OnPowerEvent(powerStatus);
        }

        /// <summary>
        /// OnSessionChange(): To handle a change event from a Terminal Server session.
        ///   Useful if you need to determine when a user logs in remotely or logs off,
        ///   or when someone logs into the console.
        /// </summary>
        /// <param name="changeDescription"></param>
        protected override void OnSessionChange(SessionChangeDescription changeDescription)
        {
            base.OnSessionChange(changeDescription);
        }
    }
}
